#!/usr/bin/python
from flask import Flask, request
from flask_restful import reqparse, Api, Resource
from dnszone import DnsZone
from ipaddress import ip_address, IPv4Address
from openid import *
import os
from dns import resolver
from mango import Mango

app = Flask(__name__)
api = Api(app)
parser = reqparse.RequestParser()

parser.add_argument('name')
parser.add_argument('value')
parser.add_argument('type')

# verander de variabelen hieronder als je de script handmatig uitvoert

dnsserver = os.environ.get('DNS_SERVER')
mongo_connect = os.environ.get('MONGO_CONNECTIONSTRING')
dnsserverport = int(os.environ.get('DNS_PORT', default=53))
debug = bool(os.environ.get('API_DEBUG', default=False))

# end change

if dnsserver is None or mongo_connect is None:
    print('You did not set DNS_SERVER or MONGO_CONNECTIONSTRING environ')
    exit(1)


def check_ipv4(ipv4):  # check if given ipv4 address in string is a valid one
    try:
        if type(ip_address(ipv4)) is not IPv4Address:  # als ip adress ongelig is
            raise ValueError  # raise value error
    except ValueError:  # geen geldige ip adres formaat
        return False

    return True  # adres correct


def make_fqdn_check(subname, parentdomain):  # make fqdn from given subdomain name, also check if valid
    fqdn = f'{subname}.{parentdomain}'
    if True:  # do some regex checking if it made a correct fqdn, currently passing and trusting user input!!!
        return fqdn
    else:
        return False


def validate_authorization(req):  # validate authorization header
    jwt = req.get('Authorization')  # get Authorization header
    if jwt is None:  # if not set
        return False

    form, value = jwt.split(' ')  # get header type and value

    if form == "Bearer":  # if bearer(openid)
        if GoogleOID.check_jwt(value)['error'] is False:  # check if jwt is valid
            return True
    elif form == "Basic":  # basic auth stored in db
        if mango.check_api_key(value):  # check if apikey exists in db
            return True
    return False


class domains(Resource):
    def get(self):  # list domains
        """
        list all domains
        :return: {"domains": ["name":"a.b"], "error":false}
        """
        # Vooralsnog moet je handmatig domains toevoegen aan BIND, dus dit heeft niet veel nut, hardcoded domeinen.
        if validate_authorization(request.headers) is False:  # check authorization
            return {"error": True, "reason": "Invalid authorization header"}, 403

        return {'domains': ['dns.mashallah.nl'], 'error': False}


class domain(Resource):
    def delete(self, dmn):
        """
        Deze functie delete een record in de dns.
        :param fqdn: domain name
        :return:
        """
        if validate_authorization(request.headers) is False:  # check authorization
            return {"error": True, "reason": "Invalid authorization header"}, 403

        args = parser.parse_args()
        t = args.get('type')
        n = args.get('name')

        if t is None or n is None:
            return {"error": True, "reason": "Required data not set"}, 400

        if t != 'A':
            return {"error": True, "reason": "Only A records are supported"}, 400

        fqdn = make_fqdn_check(n, dmn)  # combine subdomain with parent domain
        if fqdn is False:
            return {"error": True, "reason": "Invalid subdomain, domain combination"}, 400

        dns = DnsZone(dmn, dnsserver, 1, dnsserverport)  # setup dnszone

        current = dns.check_address(fqdn)  # check if address exists

        if current['error'] is True:
            if 'not found' not in current['error_text']:  # als er een error is wat NIET een "not found" is
                return {"error": True, "reason": current['error_text']}, 500
            else:  # record bestond al niet
                return {"error": True, "reason": "Record does not exist"}, 400

        new = dns.clear_address(fqdn)  # delete record

        if new['error'] is True:  # als er een error is met het weghalen van dns regel
            return {"error": True, "reason": new['error_text']}, 500

        return {'error': False, 'type': t, 'name': n}

    def get(self, dmn):
        """
        returnt een lijst met alle A records in de nameserver
        :param dmn: domain name
        :return: alle dns A records als keys
        """
        if validate_authorization(request.headers) is False:  # check authorization
            return {"error": True, "reason": "Invalid authorization header"}, 403

        dns = DnsZone(dmn, dnsserver, 1, dnsserverport)  # setup dnszone
        current = dns.list_addresses()  # get all a records

        if current['error'] is True:
            return {"error": True, "reason": current['error_text']}, 500

        return {'error': False, 'records': current['subdomains']}

    def post(self, dmn):
        """
        Maakt of past een dns A record aan in de dns server
        Als record bestaat past het de record aan, anders maakt het de record aan
        :param dmn: domain name
        :return:
        """
        if validate_authorization(request.headers) is False:  # check authorization
            return {"error": True, "reason": "Invalid authorization header"}, 403

        args = parser.parse_args()
        t = args.get('type')
        n = args.get('name')
        v = args.get('value')
        if t is None or n is None or v is None:  # check of parameters wel bestaan
            return {"error": True, "reason": "Required data not set"}, 400

        if t != 'A':  # als record type niet A is
            return {"error": True, "reason": "Only A records are supported"}, 400

        if check_ipv4(v) is False:  # check if ip is valid
            return {"error": True, "reason": "Value is not correct"}, 400

        fqdn = make_fqdn_check(n, dmn)  # maak fqdn
        if fqdn is False:  # fqdn check
            return {"error": True, "reason": "Invalid subdomain, domain combination"}, 400

        dns = DnsZone(dmn, dnsserver, 1, dnsserverport)  # no error handling because this does not throw an error

        current = dns.check_address(fqdn)  # vragen of adres bestaat

        if current['error'] is True:  # als er een error is
            if 'not found' not in current['error_text']:  # als error niet een niet bestaande record is
                return {"error": True, "reason": current['error_text']}, 500

        if current['error'] is False:  # als adres bestaat
            new = dns.update_address(fqdn, v)

            if new['error'] is True:  # als error met update adres
                return {"error": True, "reason": new['error_text']}, 500

            return {"type": t, "name": n, "value": v, "error": False}
        else:  # is True(record does not exist)
            new = dns.add_address(fqdn, v)

            if new['error'] is True:
                return {"error": True, "reason": new['error_text'], fqdn: v}, 500

            return {"type": t, "name": n, "value": v, "error": False}


api.add_resource(domains, '/api/v1/dns/domains')
api.add_resource(domain, '/api/v1/dns/domain/<dmn>')

if __name__ == '__main__':
    if check_ipv4(dnsserver) is False:  # cant use domain in resolver so check if direct ip is given if not
        dnsserver = resolver.resolve(dnsserver, "A")[0].to_text()  # try to resolve given string

    # do not setup dns zone globally because it errors on simultaneous requests
    GoogleOID = GoogleOID()  # setup google oid
    mango = Mango(mongo_connect)
    app.run(debug=debug, host='0.0.0.0', port=5001)  # run werkzeug
