from flask import Flask, render_template, redirect, request, url_for, session, flash
import secrets
import requests
from mango import Mango
from openid import *
import os

# the options below you must change if you want to use a different zone and or want to run the script manually

zone = 'dns.mashallah.nl'
dns_api = os.environ.get('DNS_API')
mongo_connection = os.environ.get('MONGO_CONNECTIONSTRING')
debug = bool(os.environ.get('API_DEBUG', default=False))  # debug option as bool

# end editing

if dns_api is None or mongo_connection is None:
    # check if vars are not set
    print('Missing DNS or MONGO environs')
    exit(1)

app = Flask(__name__)
app.secret_key = secrets.token_hex(22)


def record_update(name, typ, value, jwt):  # update record via api server
    data = {'name': name, 'type': typ, 'value': value}
    headers = {'Authorization': f'Bearer {jwt}'}

    r = requests.post(f'{dns_api}/api/v1/dns/domain/{zone}', data=data, headers=headers)

    if r.status_code != 200:  # if not sucessful
        return {'error': True, 'reason': f'Request got status code: {r.status_code}'}  # return status code from api

    return {'error': False}


def records_get(jwt):
    headers = {'Authorization': f'Bearer {jwt}'}

    r = requests.get(f'{dns_api}/api/v1/dns/domain/{zone}', headers=headers)

    if r.status_code != 200:  # if not sucessful
        return {'error': True, 'reason': f'Request got status code: {r.status_code}'}  # return status code from api
    return r.json()['records']  # return all the records as a dict


def record_delete(name, jwt):
    data = {'name': name, 'type': 'A'}
    headers = {'Authorization': f'Bearer {jwt}'}

    r = requests.delete(f'{dns_api}/api/v1/dns/domain/{zone}', data=data, headers=headers)

    if r.status_code != 200:  # if not sucessful
        return {'error': True, 'reason': f'Request got status code: {r.status_code}'}  # return status code from api

    return {'error': False}


@app.route('/')
def index():  # base index page
    return render_template('index.html')


@app.route('/login/gcp/start')
def login_start():  # client gets redirected to google login page with parameters
    nonce = secrets.token_hex(16)
    return redirect(f'https://accounts.google.com/o/oauth2/v2/auth?'
                    f'response_type=code'
                    f'&scope=openid%20profile%20email'
                    f'&redirect_uri={GoogleOID.settings["callback_uri"]}'
                    f'&client_id={GoogleOID.settings["client_id"]}'
                    f'&nonce={nonce}')


@app.route('/login/gcp/callback')
def login_callback():  # client gets returned from google with parameters
    code = request.args.get('code')  # get the code given by google

    if code is None:
        return 'Did not get correct parameters. Missing code'

    grant = GoogleOID.get_token_from_code(code)  # exchange code for access token

    if grant['error'] is True:
        return f"Could not exchange code for token: {grant['reason']}"

    profile = GoogleOID.get_profile_information(grant['data']['access_token'])  # get profile info from google

    if profile['error'] is True:
        return f"Could not get profile information: {profile['reason']}"

    if db.google_check_sso_uuid(profile['profile']['sub'])['error'] is True:  # check if user may login
        return f"Account is unavailable for login {profile['profile']['sub']}"

    db.google_update_profile(profile['profile'])  # update google profile in db
    db.google_update_lastlogin(profile['profile']['sub'])  # change login date in db
    db.google_overwrite_jwt(profile['profile']['sub'], grant['data']['id_token'])  # overwrite jwt in db

    session['username'] = profile['profile']['sub']  # set flask session so user gets logged in

    return redirect(url_for('dashboard'))  # redirect to dashboard


@app.route('/dashboard', methods=['GET', 'POST'])
def dashboard():  # dns manager dashboard
    if 'username' not in session:  # check if flask session is set(user logged in)
        return 'You are not logged in <a href="/">login</a>'

    if request.method == 'POST':  # if form posted
        na = request.form.get('name')
        ty = request.form.get('type')
        va = request.form.get('value')
        rq = request.form.get('request')

        if rq != "Add" and rq != "Delete" and rq != "Update" and rq != "Query JWT" and rq != "Add SUB":
            # data is missing
            return 'Invalid request, did you use the dashboard?'

        if rq == 'Add' or rq == 'Update':  # if user adds or updates a record
            jwt = db.google_get_jwt(session['username'])  # get jwt from db

            response = record_update(name=na, typ=ty, value=va, jwt=jwt)  # update record

            if response['error'] is True:  # if error with update record
                return f"Error processing request: {response['reason']}"

        elif rq == 'Delete':
            jwt = db.google_get_jwt(session['username'])  # get jwt from db

            response = record_delete(name=na, jwt=jwt)  # delete record

            if response['error'] is True:  # if error with delete record
                return f"Error processing request: {response['reason']}"

        elif rq == "Query JWT":  # if user requests jwt from db
            jwt = db.google_get_jwt(session['username'])  # get jwt from db
            flash(jwt)  # flash jwt message

        elif rq == "Add SUB":  # user adds a sub
            db.google_add_new_sub(va)  # add sub to db

    # the functions below are always returned even if POST is used this will keep the page the same after a POST request

    uuid = session['username']  # get uuid from session
    profile = db.google_get_profile(uuid)  # get profile from db
    lastlogin = db.google_get_lastlogin(uuid)  # get lastlogin from db
    jwt = db.google_get_jwt(uuid)  # get jwt from db
    records = records_get(jwt)  # request records from dns api

    return render_template('dashboard.html', profile=profile, records=records, lastlogin=lastlogin)


@app.route('/logout')
def logout():  # user logs out
    session.pop('username', None)  # remove flask session
    return redirect(url_for('index'))


if __name__ == '__main__':
    db = Mango(mongo_connection)  # setup mango
    GoogleOID = GoogleOID()  # initialize googleoid
    app.run(debug=debug, host='0.0.0.0')  # run werkzeug
